@@ -22,11 +22,10 @@
 
 #include "avcodec.h"
 #include "libavutil/intreadwrite.h"
+#include "libavutil/mem.h"
 #include "bytestream.h"
 #define BITSTREAM_READER_LE
 #include "get_bits.h"
-// for av_memcpy_backptr
-#include "libavutil/lzo.h"
 
 typedef struct XanContext {
     AVCodecContext *avctx;
@@ -44,12 +43,7 @@ static av_cold int xan_decode_init(AVCodecContext *avctx)
 
     s->avctx = avctx;
 
-    avctx->pix_fmt = PIX_FMT_YUV420P;
-
-    if (avctx->width & 1) {
-        av_log(avctx, AV_LOG_ERROR, "Invalid frame width: %d.\n", avctx->width);
-        return AVERROR(EINVAL);
-    }
+    avctx->pix_fmt = AV_PIX_FMT_YUV420P;
 
     s->buffer_size = avctx->width * avctx->height;
     s->y_buffer = av_malloc(s->buffer_size);
@@ -67,42 +61,42 @@ static av_cold int xan_decode_init(AVCodecContext *avctx)
 static int xan_unpack_luma(XanContext *s,
                            uint8_t *dst, const int dst_size)
 {
-   int tree_size, eof;
-   int bits, mask;
-   int tree_root, node;
-   const uint8_t *dst_end = dst + dst_size;
-   GetByteContext tree = s->gb;
-   int start_off = bytestream2_tell(&tree);
+    int tree_size, eof;
+    int bits, mask;
+    int tree_root, node;
+    const uint8_t *dst_end = dst + dst_size;
+    GetByteContext tree = s->gb;
+    int start_off = bytestream2_tell(&tree);
 
-   tree_size = bytestream2_get_byte(&s->gb);
-   eof       = bytestream2_get_byte(&s->gb);
-   tree_root = eof + tree_size;
-   bytestream2_skip(&s->gb, tree_size * 2);
+    tree_size = bytestream2_get_byte(&s->gb);
+    eof       = bytestream2_get_byte(&s->gb);
+    tree_root = eof + tree_size;
+    bytestream2_skip(&s->gb, tree_size * 2);
 
-   node = tree_root;
-   bits = bytestream2_get_byte(&s->gb);
-   mask = 0x80;
-   for (;;) {
-       int bit = !!(bits & mask);
-       mask >>= 1;
-       bytestream2_seek(&tree, start_off + node*2 + bit - eof * 2, SEEK_SET);
-       node = bytestream2_get_byte(&tree);
-       if (node == eof)
-           break;
-       if (node < eof) {
-           *dst++ = node;
-           if (dst > dst_end)
-               break;
-           node = tree_root;
-       }
-       if (!mask) {
-           if (bytestream2_get_bytes_left(&s->gb) <= 0)
-               break;
-           bits = bytestream2_get_byteu(&s->gb);
-           mask = 0x80;
-       }
-   }
-   return dst != dst_end ? AVERROR_INVALIDDATA : 0;
+    node = tree_root;
+    bits = bytestream2_get_byte(&s->gb);
+    mask = 0x80;
+    for (;;) {
+        int bit = !!(bits & mask);
+        mask >>= 1;
+        bytestream2_seek(&tree, start_off + node*2 + bit - eof * 2, SEEK_SET);
+        node = bytestream2_get_byte(&tree);
+        if (node == eof)
+            break;
+        if (node < eof) {
+            *dst++ = node;
+            if (dst > dst_end)
+                break;
+            node = tree_root;
+        }
+        if (!mask) {
+            if (bytestream2_get_bytes_left(&s->gb) <= 0)
+                break;
+            bits = bytestream2_get_byteu(&s->gb);
+            mask = 0x80;
+        }
+    }
+    return dst != dst_end ? AVERROR_INVALIDDATA : 0;
 }
 
 /* almost the same as in xan_wc3 decoder */
@@ -139,7 +133,7 @@ static int xan_unpack(XanContext *s,
             }
             if (dest + size + size2 > dest_end ||
                 dest - orig_dest + size < back)
-                return -1;
+                return AVERROR_INVALIDDATA;
             bytestream2_get_buffer(&s->gb, dest, size);
             dest += size;
             av_memcpy_backptr(dest, back, size2);
@@ -149,7 +143,7 @@ static int xan_unpack(XanContext *s,
 
             size = finish ? opcode & 3 : ((opcode & 0x1f) << 2) + 4;
             if (dest_end - dest < size)
-                return -1;
+                return AVERROR_INVALIDDATA;
             bytestream2_get_buffer(&s->gb, dest, size);
             dest += size;
             if (finish)
@@ -173,7 +167,7 @@ static int xan_decode_chroma(AVCodecContext *avctx, unsigned chroma_off)
         return 0;
     if (chroma_off + 4 >= bytestream2_get_bytes_left(&s->gb)) {
         av_log(avctx, AV_LOG_ERROR, "Invalid chroma block position\n");
-        return -1;
+        return AVERROR_INVALIDDATA;
     }
     bytestream2_seek(&s->gb, chroma_off + 4, SEEK_SET);
     mode        = bytestream2_get_le16(&s->gb);
@@ -184,7 +178,7 @@ static int xan_decode_chroma(AVCodecContext *avctx, unsigned chroma_off)
 
     if (offset >= bytestream2_get_bytes_left(&s->gb)) {
         av_log(avctx, AV_LOG_ERROR, "Invalid chroma block offset\n");
-        return -1;
+        return AVERROR_INVALIDDATA;
     }
 
     bytestream2_skip(&s->gb, offset);
@@ -192,7 +186,7 @@ static int xan_decode_chroma(AVCodecContext *avctx, unsigned chroma_off)
     dec_size = xan_unpack(s, s->scratch_buffer, s->buffer_size);
     if (dec_size < 0) {
         av_log(avctx, AV_LOG_ERROR, "Chroma unpacking failed\n");
-        return -1;
+        return AVERROR_INVALIDDATA;
     }
 
     U = s->pic.data[1];
@@ -205,7 +199,9 @@ static int xan_decode_chroma(AVCodecContext *avctx, unsigned chroma_off)
                 if (src_end - src < 1)
                     return 0;
                 val = *src++;
-                if (val && val < table_size) {
+                if (val) {
+                    if (val >= table_size)
+                        return AVERROR_INVALIDDATA;
                     val  = AV_RL16(table + (val << 1));
                     uval = (val >> 3) & 0xF8;
                     vval = (val >> 8) & 0xF8;
@@ -225,7 +221,9 @@ static int xan_decode_chroma(AVCodecContext *avctx, unsigned chroma_off)
                 if (src_end - src < 1)
                     return 0;
                 val = *src++;
-                if (val && val < table_size) {
+                if (val) {
+                    if (val >= table_size)
+                        return AVERROR_INVALIDDATA;
                     val  = AV_RL16(table + (val << 1));
                     uval = (val >> 3) & 0xF8;
                     vval = (val >> 8) & 0xF8;
@@ -258,7 +256,7 @@ static int xan_decode_frame_type0(AVCodecContext *avctx)
     if ((ret = xan_decode_chroma(avctx, chroma_off)) != 0)
         return ret;
 
-    if (corr_off >= (s->gb.buffer_end - s->gb.buffer_start)) {
+    if (corr_off >= bytestream2_size(&s->gb)) {
         av_log(avctx, AV_LOG_WARNING, "Ignoring invalid correction block position\n");
         corr_off = 0;
     }
@@ -278,7 +276,8 @@ static int xan_decode_frame_type0(AVCodecContext *avctx)
         ybuf[j+1] = cur << 1;
         last = cur;
     }
-    ybuf[j]  = last << 1;
+    if(j < avctx->width)
+        ybuf[j]  = last << 1;
     prev_buf = ybuf;
     ybuf += avctx->width;
 
@@ -291,19 +290,17 @@ static int xan_decode_frame_type0(AVCodecContext *avctx)
             ybuf[j+1] = cur << 1;
             last = cur;
         }
-        ybuf[j] = last << 1;
+        if(j < avctx->width)
+            ybuf[j] = last << 1;
         prev_buf = ybuf;
         ybuf += avctx->width;
     }
 
     if (corr_off) {
-        int corr_end, dec_size;
+        int dec_size;
 
-        corr_end = (s->gb.buffer_end - s->gb.buffer_start);
-        if (chroma_off > corr_off)
-            corr_end = chroma_off;
         bytestream2_seek(&s->gb, 8 + corr_off, SEEK_SET);
-        dec_size = xan_unpack(s, s->scratch_buffer, s->buffer_size / 2);
+        dec_size = xan_unpack(s, s->scratch_buffer, s->buffer_size);
         if (dec_size < 0)
             dec_size = 0;
         else
@@ -354,7 +351,8 @@ static int xan_decode_frame_type1(AVCodecContext *avctx)
             ybuf[j+1] = cur;
             last = cur;
         }
-        ybuf[j] = last;
+        if(j < avctx->width)
+            ybuf[j] = last;
         ybuf += avctx->width;
     }
 
@@ -371,7 +369,7 @@ static int xan_decode_frame_type1(AVCodecContext *avctx)
 }
 
 static int xan_decode_frame(AVCodecContext *avctx,
-                            void *data, int *data_size,
+                            void *data, int *got_frame,
                             AVPacket *avpkt)
 {
     XanContext *s = avctx->priv_data;
@@ -398,12 +396,12 @@ static int xan_decode_frame(AVCodecContext *avctx,
         break;
     default:
         av_log(avctx, AV_LOG_ERROR, "Unknown frame type %d\n", ftype);
-        return -1;
+        return AVERROR_INVALIDDATA;
     }
     if (ret)
         return ret;
 
-    *data_size = sizeof(AVFrame);
+    *got_frame = 1;
     *(AVFrame*)data = s->pic;
 
     return avpkt->size;
@@ -425,12 +423,11 @@ static av_cold int xan_decode_end(AVCodecContext *avctx)
 AVCodec ff_xan_wc4_decoder = {
     .name           = "xan_wc4",
     .type           = AVMEDIA_TYPE_VIDEO,
-    .id             = CODEC_ID_XAN_WC4,
+    .id             = AV_CODEC_ID_XAN_WC4,
     .priv_data_size = sizeof(XanContext),
     .init           = xan_decode_init,
     .close          = xan_decode_end,
     .decode         = xan_decode_frame,
     .capabilities   = CODEC_CAP_DR1,
-    .long_name = NULL_IF_CONFIG_SMALL("Wing Commander IV / Xxan"),
+    .long_name      = NULL_IF_CONFIG_SMALL("Wing Commander IV / Xxan"),
 };
-
